<?php
	class RandArrayClass
	{
		private $array = array();
		private $sorted = false;
		private $filepath;

		# A l'instanciation de l'objet, initialise un tableau de valeurs aléatoires
		function __construct ($size = 20, $min = 1, $max = 1000, $path = './oop_rand_array_to_file.txt') {
			$this->filepath = $path;
			for ($i=0; $i < $size; $i++) { 
				$this->array[] = rand($min, $max);
			}
		}

		# A la destruction de l'objet, sauvegarde les valeurs du tableau dans un fichier
		function __destruct () {
			if ($file = fopen($this->filepath, 'a')) {
				try {
					fputs($file, date("H:i:s") . PHP_EOL);
					for ($i=0; $i < count($this->array); $i++) { 
						fputs($file, $this->array[$i] . chr(10));
					}
				} catch (Exception $e) {
					echo 'Caught exception: ',  $e->getMessage(), "\n";
				} finally {
					fclose($file);
				}
			}
		}

		# Retourne le tableau
		public function getArray() {
			return $this->array;
		}

		# Définit le tableau
		public function setArray($array) {
			$this->array = $array;
		}

		# Calcule la moyenne des valeurs du tableau
		public function calculateAverageValue() : int {
			$int = (int)0;
			$count = count($this->array);
			for ($i=0; $i < $count; $i++) { 
				$int = $int + $this->array[$i];
			}
			$int = $int / count($this->array);
			return $int;
		}
	
		# Trouve la valeur minimale du tableau
		public function findMinimalValue() : int {
			$int = $this->array[0];
			$count = count($this->array);
			for ($i=0; $i < $count; $i++) { 
				if ($this->array[$i] < $int) {
					$int = $this->array[$i];
				}
			}
			return $int;
		}
	
		# Trouve la valeur maximale du tableau
		public function findMaximalValue() : int {
			$int = $this->array[0];
			$count = count($this->array);
			for ($i=0; $i < $count; $i++) { 
				if ($this->array[$i] > $int) {
					$int = $this->array[$i];
				}
			}
			return $int;
		}

		# Trie le tableau en ordre ascendant
		public function bubbleSort() {
			$temp = (int)0;
			$count = count($this->array);
			$swapped = (bool)true;
			while ($swapped) {
				$swapped = false;
				$count = $count - 1;
				for ($i=0; $i < $count; $i++) { 
					if ($this->array[$i] > $this->array[$i + 1]) {
						$swapped = true;
						$temp = $this->array[$i + 1];
						$this->array[$i + 1] = $this->array[$i];
						$this->array[$i] = $temp;
					}
				}
			}
			$this->sorted = true;
		}

		# Trouve l'index d'une valeur par recherche dichotomique
		# Trie le tableau s'il ne l'est pas encore
		public function searchValue($value) : int {
			if (!$this->sorted) {
				$this->bubbleSort();
			}
			$min = (int)0;
			$max = count($this->array) - 1;
			while ($min <= $max) {
				$index = floor(($min + $max) / 2);
				if ($this->array[$index] == $value) {
					return $index;
				} elseif ($this->array[$index] < $value) {
					$min = $index + 1;
				} elseif ($this->array[$index] > $value) {
					$max = $index - 1;
				}
			}
			return -1;
		}
	}
	
	function displayGenerateForm() : string {
		$string = 	
			'<form method="POST" action="oop_rand_array_to_file.php">
				<input type="number" name="size" placeholder="number of entries"/><br>
				<input type="number" name="min" placeholder="minimum values"/><br>
				<input type="number" name="max" placeholder="maximal values"/><br>
				<input type="submit" name="generate" value="Generate array"/>
			</form>';
		return $string;
	}

	function displaySearchForm($array) : string {
		$string = 	
			'<form method="POST" action="oop_rand_array_to_file.php">
			<input type="number" name="input" placeholder="value to find"/><br>
			<input type="submit" name="search" value="Search in array"/>';
		foreach($array as $value) {
			$string .= '<input type="hidden" name="array[]" value="' . htmlspecialchars($value) . '">';
		}
		$string .= '</form>';
		return $string;
	}

	function displayArray($array) : string {
		$string = "<table><tr><th>Index</th><th>Value</th></tr>";
		$count = count($array);
		for ($i=0; $i < $count; $i++) { 
			$string .= "<tr><td>" . $i . "</td>";
			$string .= "<td>" . $array[$i] . "</td></tr>";
		}
		$string .= "</table>";
		return $string;
	}
?>

<html>
	<!-- 
		trier les valeurs par tri à bulle et les écrire dans un fichier
		en programmation orientée objet
	-->
	<head>
		<title>M2GPI</title>
		<style>
			body {
				display: flex;
				flex-direction: column;
			}
			#forms {
				margin: auto;
				display: flex;
				align-items: flex-end;
				align-content: space-between;
			}
			#results {
				margin: auto;
			}
			#tables {
				margin: auto;
				display: flex;
				align-content: space-between;
			}
			#forms > form {
				margin: 0 1em 0 1em;
			}
			#tables > div {
				margin: 0 5em 0 5em;;
			}
		</style>
	</head>
	<body>
		<?php
			if (isset($_POST['search']) && $_POST['input'] && $_POST['array']) {
				$array = new RandArrayClass();
				$array->setArray($_POST['array']);
				$input = $_POST['input'];
			} elseif (isset($_POST['generate']) && $_POST['size'] && $_POST['min'] && $_POST['max']) {
				$array = new RandArrayClass($_POST['size'], $_POST['min'], $_POST['max']);
			} else {
				$array = new RandArrayClass();
			}

			echo '<h1>M2GPI2024 Algorithmie</h1>';
			echo '<div id="forms">';
			echo displayGenerateForm();
			echo displaySearchForm($array->getArray());
			echo '</div><div id="tables">';
			echo '<div><h3>Unordered</h3>';
			echo displayArray($array->getArray());
			echo '</div><div><h3>Ordered</h3>';
			$array->bubbleSort();
			echo displayArray($array->getArray());
			echo '</div><div id="results">';
			echo '<h3>Results</h3>';
			echo "<p>Average value is " . $array->calculateAverageValue() . "</p>";
			echo "<p>Smallest value is " . $array->findMinimalValue() . "</p>";
			echo "<p>Largest value is " . $array->findMaximalValue() . "</p>";
			if (isset($input)) {
				$result = $array->searchValue($input);
				if ($result > -1) {
					echo "Value " . $input . " exists at index " . $result;
				} else {
					echo "Value " . $input . " doesn't exist";
				}
			}
			echo '</div></div>';
			unset($array);
		?>
	</body>
</html>