<?php

    class MySplFileObject extends \SplFileObject {

        private $words = array();
        private $wordsCounted = false;
        private $wordsSorted = false;


        // A l'instanciation, ouvre un fichier, extrait les mots, compte leurs occurences et les trie.
        public function __construct(string $filename, string $openingMode = "r", bool $sortingMode = false) {
            parent::__construct($filename, $openingMode);
            $this->fetchWordsFromFile();
            $this->countWords();
            $this->sortWords($sortingMode);
        }


        // Retourne le tableau de mots
        public function getWords() {
            return $this->words;
        }


        // Définit le tableau de mots
        public function setWords($array) {
            $this->words = $array;
        }


        // Extrait les mots du fichier et les stocke dans le tableau
        public function fetchWordsFromFile() {
            $word = "";
            $words = array();
            $this->rewind();                                        // Retourne au début du fichier
            while (!$this->eof()) {                                 // Tant que la fin du fichier n'est pas atteinte
                $char = $this->fgetc();                                 // Récupère le caractère et fait avancer le pointeur
                if ($char !== false && ord($char) >= 192) {             // Si le caractère est encodé en UTF
                    $char .= $this->fgetc();                                // Ajouter le prochain octet au caractère
                    $word .= $char;                                         // Ajouter la chaine au mot
                } elseif ($char !== false && ctype_alpha($char)) {      // Sinon si le caractère est encodé en ASCII
                    $word .= $char;                                         // Ajouter le caractère au mot
                } elseif ($word !== "") {                               // Sinon si le caractère n'est pas alphabétique
                    $words[] = strtolower($word);                           // Ajouter le mot au tableau
                    $word = "";                                             // Vider le mot pour préparer le prochain
                }
            }
            $this->setWords($words);
        }


        // Ajoute l'occurence des mots au tableau
        public function countWords(){
            $array = [];
            foreach ($this->words as $word) {
                $array[$word] = isset($array[$word]) ? $array[$word] + 1 : 1;
            }
            $this->setWords($array);
            $this->wordsCounted = true;
        }


        // Trie le tableau par les valeurs (défaut) ou par les mots
        public function sortWords($mode = false) {
            if ($mode) {
                ksort($this->words);
            } else {
                arsort($this->words);
            }
            $this->wordsSorted = true;
        }


    }

?>